from datetime import datetime
from typing import Union, Optional, TYPE_CHECKING
from .. import utils, errors
from .payload import Payload

if TYPE_CHECKING:
    from .article import Article
    from .channel import Channel


class ArticleSuggestion:
    id: int
    channel: "Channel"
    author: dict
    payload: Payload
    is_under_moderation: bool
    is_deleted: bool
    under_moderation_reason: Optional[str]

    def __init__(self, data: dict, api):
        self.__api = api
        from .article import Article
        article = Article(data, api)
        for attr, value in article.__dict__.items():
            setattr(self, attr, value)

    def edit(self, article_data: Union[utils.ArticleBuilder, dict]) -> "ArticleSuggestion":
        if isinstance(article_data, utils.ArticleBuilder):
            article_data = article_data.build(channel_id=self.channel.id, is_suggestion=True, is_edit_mode=True)
        response = self.__api._post(f"/article/suggestion/edit/{self.id}", article_data)
        if response["code"] == 0:
            self.payload = Payload(article_data["payload"])
        else:
            raise errors.ArticleCreateEditError(response["code"])
        return self

    def delete(self) -> "ArticleSuggestion":
        response = self.__api._post(f"/article/suggestion/delete/{self.id}")
        if response["code"] == 0:
            self.is_deleted = True
        else:
            raise errors.ArticleSuggestionDeleteError(response["code"])
        return self

    def publish(self) -> "ArticleSuggestion":
        response = self.__api._post(f"/article/suggestion/publish/{self.id}")
        if response["code"] == 0:
            self.creation_date = int(datetime.now().timestamp())
        else:
            raise errors.ArticleSuggestionPublishError(response["code"])
        return self

    def set_vote(self, *args, **kwargs):
        raise NotImplementedError("Voting not available for article suggestions")

    def get_votes(self, *args, **kwargs):
        raise NotImplementedError("Getting votes not available for article suggestions")

    def get_reposts(self, *args, **kwargs):
        raise NotImplementedError("Getting reposts not available for article suggestions")

    def get_comments(self, *args, **kwargs):
        raise NotImplementedError("Getting comments not available for article suggestions")


from datetime import datetime
from typing import Optional, List, Union, Iterator, TYPE_CHECKING
from .. import enums, errors, utils
from .base import BaseModel
from .payload import Payload
from .user_vote import UserVote

if TYPE_CHECKING:
    from .channel import Channel
    from .comment import ArticleComment


class Article(BaseModel):
    id: int
    channel: "Channel"
    author: dict
    payload: Payload
    vote: enums.Vote
    repost_article: Optional["Article"]
    comment_count: int
    repost_count: int
    vote_count: int
    is_under_moderation: bool
    is_deleted: bool
    under_moderation_reason: Optional[str]
    contains_repost_article: bool

    def __init__(self, data: dict, api):
        super().__init__(data)
        self.__api = api
        from .channel import Channel
        self.channel = Channel(data["channel"], api)
        self.payload = Payload(data["payload"])
        self.creation_date: datetime = datetime.fromtimestamp(data["creation_date"])
        self.last_update_date: datetime = datetime.fromtimestamp(data["last_update_date"])
        self.repost_article = Article(data["repost_article"], self.__api) if data.get("repost_article") else None
        self.vote = enums.Vote(data["vote"]) if data.get("vote") else None

    def edit(self, article_data: Union[utils.ArticleBuilder, dict], repost_article_id: Optional[int] = None) -> "Article":
        if isinstance(article_data, utils.ArticleBuilder):
            article_data = article_data.build(channel_id=self.channel.id, is_edit_mode=True)
        if repost_article_id is not None:
            article_data["repost_article_id"] = repost_article_id
        response = self.__api._post(f"/article/edit/{self.id}", article_data)
        if response["code"] == 0:
            self.payload = Payload(article_data["payload"])
            self.repost_article = Article(response["article"]["repost_article"], self.__api) if response["article"]["repost_article"] else None
        else:
            raise errors.ArticleCreateEditError(response["code"])
        return self

    def delete(self) -> "Article":
        response = self.__api._post(f"/article/delete/{self.id}")
        if response["code"] == 0:
            self.is_deleted = True
        else:
            raise errors.AnixartError(response["code"], "Статья не найдена.")
        return self
    
    def set_vote(self, vote: Union[enums.Vote, int]) -> dict:
        response = self.__api._get(f"/article/vote/{self.id}/{vote}")
        if response["code"] == 0:
            self.vote = enums.Vote(vote)
        else:
            raise errors.DefaultError(response["code"])
        return response

    def _fetch_votes_page(self, sort: enums.Sorting, page: int) -> tuple[list[UserVote], int]:
        response = self.__api._get(f"/article/votes/{self.id}/{page}?sort={sort}")
        if response["code"] == 0:
            items = [UserVote(vote) for vote in response["content"]]
            return items, response["total_page_count"]
        raise errors.AnixartError(response["code"], "Ошибка загрузки голосов")

    def get_votes(self, sort: enums.Sorting, page: Union[int, range, None] = None) -> Union[list[UserVote], Iterator[UserVote]]:
        return utils.paginate(lambda pg: self._fetch_votes_page(sort, pg), page)

    def _fetch_reposts_page(self, sort: enums.Sorting, page: int) -> tuple[list["Article"], int]:
        response = self.__api._get(f"/article/reposts/{self.id}/{page}?sort={sort}")
        if response["code"] == 0:
            items = [Article(repost, self.__api) for repost in response["content"]]
            return items, response["total_page_count"]
        raise errors.AnixartError(response["code"], "Ошибка загрузки репостов")

    def get_reposts(self, sort: enums.Sorting, page: Union[int, range, None] = None) -> Union[list["Article"], Iterator["Article"]]:
        return utils.paginate(lambda pg: self._fetch_reposts_page(sort, pg), page)
    
    def _fetch_comments_page(self, sort: enums.Sorting, page: int) -> tuple[list["ArticleComment"], int]:
        from .comment import ArticleComment
        response = self.__api._get(f"/article/comment/all/{self.id}/{page}?sort={sort}")
        if response["code"] == 0:
            items = [ArticleComment(comment, self.__api) for comment in response["content"]]
            return items, response["total_page_count"]
        raise errors.AnixartError(response["code"], "Ошибка загрузки комментариев")
    
    def get_comments(self, sort: enums.Sorting, page: Union[int, range, None] = 0) -> Union[list["ArticleComment"], Iterator["ArticleComment"]]:
        return utils.paginate(lambda pg: self._fetch_comments_page(sort, pg), page)


import time
import uuid
from . import anix_images
from typing import Union, Optional, Iterator, Callable, Any, List


class Style:
    @staticmethod
    def bold(text: str) -> str:
        return f"<b>{text}</b>"

    @staticmethod
    def underline(text: str) -> str:
        return f"<u>{text}</u>"

    @staticmethod
    def italic(text: str) -> str:
        return f"<i>{text}</i>"

    @staticmethod
    def strike(text: str) -> str:
        return f"<s>{text}</s>"

    @staticmethod
    def link(text: str, url: str) -> str:
        return f'<a href="{url}">{text}</a>'


class ArticleBuilder:
    EDITOR_VERSION = "2.26.5"

    def __init__(self, channel_id: Optional[int] = None, request_delay: float = 0.25):
        self.channel_id = channel_id
        self.request_delay = request_delay
        self.payload = {
            "time": int(time.time() * 1000),
            "blocks": [],
            "version": self.EDITOR_VERSION,
            "block_count": 0
        }
        self._media_files = []
        self._embed_links = []
    
    def _add_block(self, name, block_type, data):
        block = {"id": str(uuid.uuid4())[:12], "name": name, "type": block_type, "data": data}
        self.payload["blocks"].append(block)
        self.payload["block_count"] += 1
        return self
    
    def add_header(self, text: str, level: int = 3):
        return self._add_block("header", "header", {"text": text, "level": level, "text_length": len(text)})
    
    def add_paragraph(self, text: str):
        return self._add_block("paragraph", "paragraph", {"text": text, "text_length": len(text)})
    
    def add_quote(self, text: str, caption: str | None = None, alignment: str = "left"):
        return self._add_block("quote", "quote", {"text": text, "caption": caption, "alignment": alignment, "text_length": len(text), "caption_length": len(caption or "")})
    
    def add_delimiter(self):
        return self._add_block("delimiter", "delimiter", {})
    
    def add_list(self, items: list, ordered: bool = False):
        return self._add_block("list", "list", {"items": items, "style": ("un", "")[ordered] + "ordered", "item_count": len(items)})
    
    def add_media(self, files: Union[str, List[str]]):
        if isinstance(files, str):
            files = [files]
        self._media_files.extend(files)
        return self._add_block("media", "media", {"items": [{"url": f"pending_{i}"} for i in range(len(self._media_files))], "item_count": len(self._media_files)})
    
    def add_embed(self, link: str):
        self._embed_links.append(link)
        return self._add_block("embed", "embed", {"service": "pending", "url": link})
    
    def _upload_media(self, is_suggestion: bool = False, is_edit_mode: bool = False):
        """Загружает все отложенные медиафайлы"""
        if (self._media_files or self._embed_links) and not self.channel_id:
            raise ValueError("ArticleBuilder(channel_id) обязателен для загрузки медиа и вложений")

        media_results = []
        if self._media_files:
            media_results = anix_images.upload_media_files(
                self.channel_id,
                self._media_files,
                is_suggestion,
                is_edit_mode,
                self.request_delay
            )
        
        embed_results = []
        if self._embed_links:
            for link in self._embed_links:
                result = anix_images.upload_embed_content(
                    self.channel_id,
                    link,
                    is_suggestion,
                    is_edit_mode
                )
                embed_results.append(result)
                if self.request_delay > 0:
                    time.sleep(self.request_delay)
        
        return media_results, embed_results
    
    def build(self, channel_id: Optional[int] = None, is_suggestion=False, is_edit_mode=False):
        """Собирает финальный payload с загрузкой медиа и вложений"""
        self.channel_id = channel_id
        media_results, embed_results = self._upload_media(is_suggestion, is_edit_mode)
        
        embed_index = 0
        for block in self.payload["blocks"]:
            if block["type"] == "media":
                block["data"]["items"] = [r["file"] for r in media_results]
            elif block["type"] == "embed":
                block["data"].update(embed_results[embed_index])
                embed_index += 1
        
        return {"payload": self.payload}


class Paginator:
    """Универсальный пагинатор для любых объектов."""
    
    def __init__(
        self,
        fetch_func: Callable[[int], Any],
        start_page: int = 0,
        end_page: Optional[int] = None
    ):
        self.fetch_func = fetch_func
        self.start_page = start_page
        self.end_page = end_page
        self._current_page = start_page - 1  # Для __next__
        self._total_pages = None
        self._buffer = []
        self._buffer_index = 0

    def __iter__(self) -> Iterator[Any]:
        return self

    def __next__(self) -> Any:
        while True:
            # Если в буфере есть элементы — возвращаем их
            if self._buffer_index < len(self._buffer):
                item = self._buffer[self._buffer_index]
                self._buffer_index += 1
                return item

            # Переход к следующей странице
            self._current_page += 1

            # Проверка на выход за границы
            if self._total_pages is not None and self._current_page >= self._total_pages:
                raise StopIteration
            if self.end_page is not None and self._current_page > self.end_page:
                raise StopIteration

            # Загрузка данных
            try:
                items, total_pages = self.fetch_func(self._current_page)
            except Exception as e:
                raise StopIteration from e

            # Обновление общего числа страниц (если ещё не известно)
            if self._total_pages is None:
                self._total_pages = total_pages
                # Корректируем end_page, если он не задан или превышает общее кол-во
                if self.end_page is None:
                    self.end_page = self._total_pages - 1
                else:
                    self.end_page = min(self.end_page, self._total_pages - 1)

            # Если страница пуста или вышли за границы — завершаем итерацию
            if not items:
                raise StopIteration

            # Обновляем буфер
            self._buffer = items
            self._buffer_index = 0


def paginate(
    fetch_func: Callable[[int], tuple[list[Any], int]],
    page: Union[int, range, None] = None
) -> Union[list[Any], Paginator]:
    """Универсальная функция для пагинации.
    
    Args:
        fetch_func: Функция, которая принимает номер страницы и возвращает (items, total_pages).
        page: 
            - int: загрузить только эту страницу.
            - range: загрузить страницы из диапазона.
            - None: загрузить все страницы (от 0 до последней).
    
    Returns:
        - Если page=int → возвращает список элементов.
        - Если page=range или None → возвращает Paginator (итерируемый объект).
    """
    if isinstance(page, int):
        items, _ = fetch_func(page)
        return items
    else:
        start = 0 if page is None else (page.start if isinstance(page, range) else 0)
        end = None if page is None else (page.stop - 1 if isinstance(page, range) else None)
        return Paginator(fetch_func, start_page=start, end_page=end)